@@ -21,7 +21,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <gudev/gudev.h>
 
 #include <ModemManager.h>
 #include <mm-errors-types.h>
@@ -47,6 +46,7 @@ enum {
     PROP_VENDOR_ID,
     PROP_PRODUCT_ID,
     PROP_CONNECTION,
+    PROP_REPROBE,
     PROP_LAST
 };
 
@@ -70,6 +70,7 @@ struct _MMBaseModemPrivate {
 
     gboolean hotplugged;
     gboolean valid;
+    gboolean reprobe;
 
     guint max_timeouts;
 
@@ -88,6 +89,10 @@ struct _MMBaseModemPrivate {
     MMPortSerialAt *gps_control;
     MMPortSerialGps *gps;
 
+    /* Support for parallel enable/disable operations */
+    GList *enable_tasks;
+    GList *disable_tasks;
+
 #if defined WITH_QMI
     /* QMI ports */
     GList *qmi;
@@ -109,36 +114,46 @@ get_hash_key (const gchar *subsys,
 static void
 serial_port_timed_out_cb (MMPortSerial *port,
                           guint n_consecutive_timeouts,
-                          gpointer user_data)
+                          MMBaseModem *self)
 {
-    MMBaseModem *self = (MM_BASE_MODEM (user_data));
-
-    if (self->priv->max_timeouts > 0 &&
-        n_consecutive_timeouts >= self->priv->max_timeouts) {
-        mm_warn ("(%s/%s) port timed out %u times, marking modem '%s' as disabled",
-                 mm_port_type_get_string (mm_port_get_port_type (MM_PORT (port))),
+    /* If reached the maximum number of timeouts, invalidate modem */
+    if (n_consecutive_timeouts >= self->priv->max_timeouts) {
+        mm_err ("(%s/%s) %s port timed out %u consecutive times, marking modem '%s' as invalid",
+                 mm_port_subsys_get_string (mm_port_get_subsys (MM_PORT (port))),
                  mm_port_get_device (MM_PORT (port)),
+                 mm_port_type_get_string (mm_port_get_port_type (MM_PORT (port))),
                  n_consecutive_timeouts,
                  g_dbus_object_get_object_path (G_DBUS_OBJECT (self)));
-
-        /* Only set action to invalidate modem if not already done */
         g_cancellable_cancel (self->priv->cancellable);
+        return;
     }
+
+    if (n_consecutive_timeouts > 1)
+        mm_warn ("(%s/%s) %s port timed out %u consecutive times",
+                 mm_port_subsys_get_string (mm_port_get_subsys (MM_PORT (port))),
+                 mm_port_get_device (MM_PORT (port)),
+                 mm_port_type_get_string (mm_port_get_port_type (MM_PORT (port))),
+                 n_consecutive_timeouts);
 }
 
 gboolean
-mm_base_modem_grab_port (MMBaseModem *self,
-                         const gchar *subsys,
-                         const gchar *name,
-                         const gchar *parent_path,
-                         MMPortType ptype,
-                         MMPortSerialAtFlag at_pflags,
-                         GError **error)
+mm_base_modem_grab_port (MMBaseModem         *self,
+                         MMKernelDevice      *kernel_device,
+                         MMPortType           ptype,
+                         MMPortSerialAtFlag   at_pflags,
+                         GError             **error)
 {
-    MMPort *port;
-    gchar *key;
+    MMPort      *port;
+    gchar       *key;
+    const gchar *subsys;
+    const gchar *name;
 
     g_return_val_if_fail (MM_IS_BASE_MODEM (self), FALSE);
+    g_return_val_if_fail (MM_IS_KERNEL_DEVICE (kernel_device), FALSE);
+
+    subsys = mm_kernel_device_get_subsystem (kernel_device);
+    name   = mm_kernel_device_get_name      (kernel_device);
+
     g_return_val_if_fail (subsys != NULL, FALSE);
     g_return_val_if_fail (name != NULL, FALSE);
 
@@ -208,11 +223,18 @@ mm_base_modem_grab_port (MMBaseModem *self,
             return FALSE;
         }
 
-        /* For serial ports, enable port timeout checks */
-        g_signal_connect (port,
-                          "timed-out",
-                          G_CALLBACK (serial_port_timed_out_cb),
-                          self);
+        /* For serial ports, enable port timeout checks if requested to do so */
+        if (self->priv->max_timeouts > 0)
+            g_signal_connect (port,
+                              "timed-out",
+                              G_CALLBACK (serial_port_timed_out_cb),
+                              self);
+
+        /* For serial ports, optionally use a specific baudrate */
+        if (mm_kernel_device_has_property (kernel_device, "ID_MM_TTY_BAUDRATE"))
+            g_object_set (port,
+                          MM_PORT_SERIAL_BAUD, mm_kernel_device_get_property_as_int (kernel_device, "ID_MM_TTY_BAUDRATE"),
+                          NULL);
     }
     /* Net ports... */
     else if (g_str_equal (subsys, "net")) {
@@ -272,7 +294,7 @@ mm_base_modem_grab_port (MMBaseModem *self,
     }
     else
         /* We already filter out before all non-tty, non-net, non-cdc-wdm ports */
-        g_assert_not_reached();
+        g_assert_not_reached ();
 
     mm_dbg ("(%s) type '%s' claimed by %s",
             name,
@@ -283,58 +305,132 @@ mm_base_modem_grab_port (MMBaseModem *self,
      * Note: 'key' and 'port' now owned by the HT. */
     g_hash_table_insert (self->priv->ports, key, port);
 
-    /* Store parent path */
+    /* Store kernel device */
     g_object_set (port,
-                  MM_PORT_PARENT_PATH, parent_path,
+                  MM_PORT_KERNEL_DEVICE, kernel_device,
                   NULL);
 
     return TRUE;
 }
 
 gboolean
-mm_base_modem_disable_finish (MMBaseModem *self,
-                              GAsyncResult *res,
-                              GError **error)
+mm_base_modem_disable_finish (MMBaseModem   *self,
+                              GAsyncResult  *res,
+                              GError       **error)
 {
-    return MM_BASE_MODEM_GET_CLASS (self)->disable_finish (self, res, error);
+    return g_task_propagate_boolean (G_TASK (res), error);
+}
+
+static void
+disable_ready (MMBaseModem  *self,
+               GAsyncResult *res)
+{
+    GError *error = NULL;
+    GList  *l;
+    GList  *disable_tasks;
+
+    g_assert (self->priv->disable_tasks);
+    disable_tasks = self->priv->disable_tasks;
+    self->priv->disable_tasks = NULL;
+
+    MM_BASE_MODEM_GET_CLASS (self)->disable_finish (self, res, &error);
+    for (l = disable_tasks; l; l = g_list_next (l)) {
+        if (error)
+            g_task_return_error (G_TASK (l->data), g_error_copy (error));
+        else
+            g_task_return_boolean (G_TASK (l->data), TRUE);
+    }
+    g_clear_error (&error);
+
+    g_list_free_full (disable_tasks, (GDestroyNotify)g_object_unref);
 }
 
 void
-mm_base_modem_disable (MMBaseModem *self,
-                      GAsyncReadyCallback callback,
-                      gpointer user_data)
+mm_base_modem_disable (MMBaseModem         *self,
+                       GAsyncReadyCallback  callback,
+                       gpointer             user_data)
 {
+    GTask    *task;
+    gboolean  run_disable;
+
     g_assert (MM_BASE_MODEM_GET_CLASS (self)->disable != NULL);
     g_assert (MM_BASE_MODEM_GET_CLASS (self)->disable_finish != NULL);
 
+    /* If the list of disable tasks is empty, we need to run */
+    run_disable = !self->priv->disable_tasks;
+
+    /* Store task */
+    task = g_task_new (self, self->priv->cancellable, callback, user_data);
+    self->priv->disable_tasks = g_list_append (self->priv->disable_tasks, task);
+
+    if (!run_disable)
+        return;
+
     MM_BASE_MODEM_GET_CLASS (self)->disable (
         self,
         self->priv->cancellable,
-        callback,
-        user_data);
+        (GAsyncReadyCallback) disable_ready,
+        NULL);
 }
 
 gboolean
-mm_base_modem_enable_finish (MMBaseModem *self,
-                             GAsyncResult *res,
-                             GError **error)
+mm_base_modem_enable_finish (MMBaseModem   *self,
+                             GAsyncResult  *res,
+                             GError       **error)
 {
-    return MM_BASE_MODEM_GET_CLASS (self)->enable_finish (self, res, error);
+    return g_task_propagate_boolean (G_TASK (res), error);
+}
+
+static void
+enable_ready (MMBaseModem  *self,
+              GAsyncResult *res)
+{
+    GError *error = NULL;
+    GList  *l;
+    GList  *enable_tasks;
+
+    g_assert (self->priv->enable_tasks);
+    enable_tasks = self->priv->enable_tasks;
+    self->priv->enable_tasks = NULL;
+
+    MM_BASE_MODEM_GET_CLASS (self)->enable_finish (self, res, &error);
+    for (l = enable_tasks; l; l = g_list_next (l)) {
+        if (error)
+            g_task_return_error (G_TASK (l->data), g_error_copy (error));
+        else
+            g_task_return_boolean (G_TASK (l->data), TRUE);
+    }
+    g_clear_error (&error);
+
+    g_list_free_full (enable_tasks, (GDestroyNotify)g_object_unref);
 }
 
 void
-mm_base_modem_enable (MMBaseModem *self,
-                      GAsyncReadyCallback callback,
-                      gpointer user_data)
+mm_base_modem_enable (MMBaseModem         *self,
+                      GAsyncReadyCallback  callback,
+                      gpointer             user_data)
 {
+    GTask    *task;
+    gboolean  run_enable;
+
     g_assert (MM_BASE_MODEM_GET_CLASS (self)->enable != NULL);
     g_assert (MM_BASE_MODEM_GET_CLASS (self)->enable_finish != NULL);
 
+    /* If the list of enable tasks is empty, we need to run */
+    run_enable = !self->priv->enable_tasks;
+
+    /* Store task */
+    task = g_task_new (self, self->priv->cancellable, callback, user_data);
+    self->priv->enable_tasks = g_list_append (self->priv->enable_tasks, task);
+
+    if (!run_enable)
+        return;
+
     MM_BASE_MODEM_GET_CLASS (self)->enable (
         self,
         self->priv->cancellable,
-        callback,
-        user_data);
+        (GAsyncReadyCallback) enable_ready,
+        NULL);
 }
 
 gboolean
@@ -392,6 +488,23 @@ mm_base_modem_set_valid (MMBaseModem *self,
     }
 }
 
+void
+mm_base_modem_set_reprobe (MMBaseModem *self,
+                           gboolean reprobe)
+{
+    g_return_if_fail (MM_IS_BASE_MODEM (self));
+
+    self->priv->reprobe = reprobe;
+}
+
+gboolean
+mm_base_modem_get_reprobe (MMBaseModem *self)
+{
+    g_return_val_if_fail (MM_IS_BASE_MODEM (self), FALSE);
+
+    return self->priv->reprobe;
+}
+
 gboolean
 mm_base_modem_get_valid (MMBaseModem *self)
 {
@@ -534,9 +647,10 @@ mm_base_modem_peek_port_qmi_for_data (MMBaseModem *self,
 {
     GList *cdc_wdm_qmi_ports, *l;
     const gchar *net_port_parent_path;
+    MMPortQmi *found = NULL;
 
     g_warn_if_fail (mm_port_get_subsys (data) == MM_PORT_SUBSYS_NET);
-    net_port_parent_path = mm_port_get_parent_path (data);
+    net_port_parent_path = mm_kernel_device_get_interface_sysfs_path (mm_port_peek_kernel_device (data));
     if (!net_port_parent_path) {
         g_set_error (error,
                      MM_CORE_ERROR,
@@ -551,21 +665,25 @@ mm_base_modem_peek_port_qmi_for_data (MMBaseModem *self,
                                                   MM_PORT_SUBSYS_USB,
                                                   MM_PORT_TYPE_QMI,
                                                   NULL);
-    for (l = cdc_wdm_qmi_ports; l; l = g_list_next (l)) {
+    for (l = cdc_wdm_qmi_ports; l && !found; l = g_list_next (l)) {
         const gchar *wdm_port_parent_path;
 
         g_assert (MM_IS_PORT_QMI (l->data));
-        wdm_port_parent_path = mm_port_get_parent_path (MM_PORT (l->data));
+        wdm_port_parent_path = mm_kernel_device_get_interface_sysfs_path (mm_port_peek_kernel_device (MM_PORT (l->data)));
         if (wdm_port_parent_path && g_str_equal (wdm_port_parent_path, net_port_parent_path))
-            return MM_PORT_QMI (l->data);
+            found = MM_PORT_QMI (l->data);
     }
 
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_NOT_FOUND,
-                 "Couldn't find associated QMI port for 'net/%s'",
-                 mm_port_get_device (data));
-    return NULL;
+    g_list_free_full (cdc_wdm_qmi_ports, g_object_unref);
+
+    if (!found)
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_NOT_FOUND,
+                     "Couldn't find associated QMI port for 'net/%s'",
+                     mm_port_get_device (data));
+
+    return found;
 }
 
 #endif /* WITH_QMI */
@@ -608,9 +726,10 @@ mm_base_modem_peek_port_mbim_for_data (MMBaseModem *self,
 {
     GList *cdc_wdm_mbim_ports, *l;
     const gchar *net_port_parent_path;
+    MMPortMbim *found = NULL;
 
     g_warn_if_fail (mm_port_get_subsys (data) == MM_PORT_SUBSYS_NET);
-    net_port_parent_path = mm_port_get_parent_path (data);
+    net_port_parent_path = mm_kernel_device_get_interface_sysfs_path (mm_port_peek_kernel_device (data));
     if (!net_port_parent_path) {
         g_set_error (error,
                      MM_CORE_ERROR,
@@ -625,21 +744,26 @@ mm_base_modem_peek_port_mbim_for_data (MMBaseModem *self,
                                                   MM_PORT_SUBSYS_USB,
                                                   MM_PORT_TYPE_MBIM,
                                                   NULL);
-    for (l = cdc_wdm_mbim_ports; l; l = g_list_next (l)) {
+
+    for (l = cdc_wdm_mbim_ports; l && !found; l = g_list_next (l)) {
         const gchar *wdm_port_parent_path;
 
         g_assert (MM_IS_PORT_MBIM (l->data));
-        wdm_port_parent_path = mm_port_get_parent_path (MM_PORT (l->data));
+        wdm_port_parent_path = mm_kernel_device_get_interface_sysfs_path (mm_port_peek_kernel_device (MM_PORT (l->data)));
         if (wdm_port_parent_path && g_str_equal (wdm_port_parent_path, net_port_parent_path))
-            return MM_PORT_MBIM (l->data);
+            found = MM_PORT_MBIM (l->data);
     }
 
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_NOT_FOUND,
-                 "Couldn't find associated MBIM port for 'net/%s'",
-                 mm_port_get_device (data));
-    return NULL;
+    g_list_free_full (cdc_wdm_mbim_ports, g_object_unref);
+
+    if (!found)
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_NOT_FOUND,
+                     "Couldn't find associated MBIM port for 'net/%s'",
+                     mm_port_get_device (data));
+
+    return found;
 }
 
 #endif /* WITH_MBIM */
@@ -677,13 +801,9 @@ mm_base_modem_peek_best_data_port (MMBaseModem *self,
 GList *
 mm_base_modem_get_data_ports (MMBaseModem *self)
 {
-    GList *copy;
-
     g_return_val_if_fail (MM_IS_BASE_MODEM (self), NULL);
 
-    copy = g_list_copy (self->priv->data);
-    g_list_foreach (copy, (GFunc)g_object_ref, NULL);
-    return copy;
+    return g_list_copy_deep (self->priv->data, (GCopyFunc)g_object_ref, NULL);
 }
 
 GList *
@@ -1143,23 +1263,22 @@ mm_base_modem_authorize_finish (MMBaseModem *self,
                                 GAsyncResult *res,
                                 GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 authorize_ready (MMAuthProvider *authp,
                  GAsyncResult *res,
-                 GSimpleAsyncResult *simple)
+                 GTask *task)
 {
     GError *error = NULL;
 
     if (!mm_auth_provider_authorize_finish (authp, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
+        g_task_return_boolean (task, TRUE);
 
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 void
@@ -1169,18 +1288,14 @@ mm_base_modem_authorize (MMBaseModem *self,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
 
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        mm_base_modem_authorize);
+    task = g_task_new (self, self->priv->authp_cancellable, callback, user_data);
 
     /* When running in the session bus for tests, default to always allow */
     if (mm_context_get_test_session ()) {
-        g_simple_async_result_set_op_res_gboolean (result, TRUE);
-        g_simple_async_result_complete_in_idle (result);
-        g_object_unref (result);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -1189,7 +1304,7 @@ mm_base_modem_authorize (MMBaseModem *self,
                                 authorization,
                                 self->priv->authp_cancellable,
                                 (GAsyncReadyCallback)authorize_ready,
-                                result);
+                                task);
 }
 
 /*****************************************************************************/
@@ -1295,6 +1410,9 @@ set_property (GObject *object,
     case PROP_VALID:
         mm_base_modem_set_valid (self, g_value_get_boolean (value));
         break;
+    case PROP_REPROBE:
+        mm_base_modem_set_reprobe (self, g_value_get_boolean (value));
+        break;
     case PROP_MAX_TIMEOUTS:
         self->priv->max_timeouts = g_value_get_uint (value);
         break;
@@ -1338,6 +1456,9 @@ get_property (GObject *object,
     case PROP_VALID:
         g_value_set_boolean (value, self->priv->valid);
         break;
+    case PROP_REPROBE:
+        g_value_set_boolean (value, self->priv->reprobe);
+        break;
     case PROP_MAX_TIMEOUTS:
         g_value_set_uint (value, self->priv->max_timeouts);
         break;
@@ -1374,6 +1495,9 @@ finalize (GObject *object)
      * mm_auth_provider_cancel_for_owner (self->priv->authp, object);
     */
 
+    g_assert (!self->priv->enable_tasks);
+    g_assert (!self->priv->disable_tasks);
+
     mm_dbg ("Modem (%s) '%s' completely disposed",
             self->priv->plugin,
             self->priv->device);
@@ -1513,4 +1637,12 @@ mm_base_modem_class_init (MMBaseModemClass *klass)
                              G_TYPE_DBUS_CONNECTION,
                              G_PARAM_READWRITE);
     g_object_class_install_property (object_class, PROP_CONNECTION, properties[PROP_CONNECTION]);
+
+    properties[PROP_REPROBE] =
+        g_param_spec_boolean (MM_BASE_MODEM_REPROBE,
+                              "Reprobe",
+                              "Whether the modem needs to be reprobed or not.",
+                              FALSE,
+                              G_PARAM_READWRITE);
+    g_object_class_install_property (object_class, PROP_REPROBE, properties[PROP_REPROBE]);
 }
