@@ -29,7 +29,7 @@
 #include "mm-base-call.h"
 #include "mm-log.h"
 
-G_DEFINE_TYPE (MMCallList, mm_call_list, G_TYPE_OBJECT);
+G_DEFINE_TYPE (MMCallList, mm_call_list, G_TYPE_OBJECT)
 
 enum {
     PROP_0,
@@ -67,8 +67,7 @@ mm_call_list_get_paths (MMCallList *self)
     GList *l;
     guint i;
 
-    path_list = g_new0 (gchar *,
-                        1 + g_list_length (self->priv->list));
+    path_list = g_new0 (gchar *, 1 + g_list_length (self->priv->list));
 
     for (i = 0, l = self->priv->list; l; l = g_list_next (l)) {
         const gchar *path;
@@ -84,28 +83,27 @@ mm_call_list_get_paths (MMCallList *self)
 
 /*****************************************************************************/
 
-MMBaseCall* mm_call_list_get_new_incoming(MMCallList *self)
+MMBaseCall *
+mm_call_list_get_new_incoming (MMCallList *self)
 {
     MMBaseCall *call = NULL;
     GList *l;
-    guint i;
 
-    for (i = 0, l = self->priv->list; l && !call; l = g_list_next (l)) {
+    for (l = self->priv->list; l; l = g_list_next (l)) {
 
-        MMCallState         state;
-        MMCallStateReason   reason;
-        MMCallDirection     direct;
+        MMCallState       state;
+        MMCallStateReason reason;
+        MMCallDirection   direct;
 
         g_object_get (MM_BASE_CALL (l->data),
-                      "state"       , &state,
+                      "state",        &state,
                       "state-reason", &reason,
-                      "direction"   , &direct,
+                      "direction",    &direct,
                       NULL);
 
-        if( direct  == MM_CALL_DIRECTION_INCOMING           &&
-            state   == MM_CALL_STATE_RINGING_IN             &&
-            reason  == MM_CALL_STATE_REASON_INCOMING_NEW    ) {
-
+        if (direct  == MM_CALL_DIRECTION_INCOMING &&
+            state   == MM_CALL_STATE_RINGING_IN &&
+            reason  == MM_CALL_STATE_REASON_INCOMING_NEW ) {
             call = MM_BASE_CALL (l->data);
             break;
         }
@@ -114,23 +112,21 @@ MMBaseCall* mm_call_list_get_new_incoming(MMCallList *self)
     return call;
 }
 
-MMBaseCall* mm_call_list_get_first_ringing_call(MMCallList *self)
+MMBaseCall *
+mm_call_list_get_first_ringing_call (MMCallList *self)
 {
     MMBaseCall *call = NULL;
     GList *l;
-    guint i;
 
-    for (i = 0, l = self->priv->list; l && !call; l = g_list_next (l)) {
-
-        MMCallState         state;
+    for (l = self->priv->list; l; l = g_list_next (l)) {
+        MMCallState state;
 
         g_object_get (MM_BASE_CALL (l->data),
-                      "state"       , &state,
+                      "state", &state,
                       NULL);
 
-        if( state == MM_CALL_STATE_RINGING_IN   ||
-            state == MM_CALL_STATE_RINGING_OUT  ) {
-
+        if (state == MM_CALL_STATE_RINGING_IN ||
+            state == MM_CALL_STATE_RINGING_OUT) {
             call = MM_BASE_CALL (l->data);
             break;
         }
@@ -139,25 +135,23 @@ MMBaseCall* mm_call_list_get_first_ringing_call(MMCallList *self)
     return call;
 }
 
-MMBaseCall* mm_call_list_get_first_outgoing_dialing_call(MMCallList *self)
+MMBaseCall *
+mm_call_list_get_first_outgoing_dialing_call (MMCallList *self)
 {
     MMBaseCall *call = NULL;
     GList *l;
-    guint i;
 
-    for (i = 0, l = self->priv->list; l && !call; l = g_list_next (l)) {
-
-        MMCallState         state;
-        MMCallDirection     direct;
+    for (l = self->priv->list; l; l = g_list_next (l)) {
+        MMCallState     state;
+        MMCallDirection direction;
 
         g_object_get (MM_BASE_CALL (l->data),
-                      "state"       , &state,
-                      "direction"   , &direct,
+                      "state",     &state,
+                      "direction", &direction,
                       NULL);
 
-        if( direct == MM_CALL_DIRECTION_OUTGOING    &&
-            state  == MM_CALL_STATE_DIALING         ) {
-
+        if (direction == MM_CALL_DIRECTION_OUTGOING &&
+            state  == MM_CALL_STATE_DIALING) {
             call = MM_BASE_CALL (l->data);
             break;
         }
@@ -166,21 +160,20 @@ MMBaseCall* mm_call_list_get_first_outgoing_dialing_call(MMCallList *self)
     return call;
 }
 
-MMBaseCall* mm_call_list_get_first_non_terminated_call(MMCallList *self)
+MMBaseCall *
+mm_call_list_get_first_non_terminated_call (MMCallList *self)
 {
     MMBaseCall *call = NULL;
     GList *l;
-    guint i;
 
-    for (i = 0, l = self->priv->list; l && !call; l = g_list_next (l)) {
-
-        MMCallState         state;
+    for (l = self->priv->list; l; l = g_list_next (l)) {
+        MMCallState state;
 
         g_object_get (MM_BASE_CALL (l->data),
-                      "state"       , &state,
+                      "state", &state,
                       NULL);
 
-        if( state != MM_CALL_STATE_TERMINATED ) {
+        if (state != MM_CALL_STATE_TERMINATED) {
             call = MM_BASE_CALL (l->data);
             break;
         }
@@ -189,23 +182,23 @@ MMBaseCall* mm_call_list_get_first_non_terminated_call(MMCallList *self)
     return call;
 }
 
-gboolean mm_call_list_send_dtmf_to_active_calls(MMCallList *self, gchar *dtmf)
+gboolean
+mm_call_list_send_dtmf_to_active_calls (MMCallList  *self,
+                                        const gchar *dtmf)
 {
     gboolean signaled = FALSE;
     GList *l;
-    guint i;
 
-    for (i = 0, l = self->priv->list; l; l = g_list_next (l)) {
-
-        MMCallState         state;
+    for (l = self->priv->list; l; l = g_list_next (l)) {
+        MMCallState state;
 
         g_object_get (MM_BASE_CALL (l->data),
-                      "state"       , &state,
+                      "state", &state,
                       NULL);
 
-        if( state == MM_CALL_STATE_ACTIVE ) {
+        if (state == MM_CALL_STATE_ACTIVE) {
             signaled = TRUE;
-            mm_base_call_received_dtmf(MM_BASE_CALL (l->data), dtmf);
+            mm_base_call_received_dtmf (MM_BASE_CALL (l->data), dtmf);
         }
     }
 
@@ -214,28 +207,12 @@ gboolean mm_call_list_send_dtmf_to_active_calls(MMCallList *self, gchar *dtmf)
 
 /*****************************************************************************/
 
-typedef struct {
-    MMCallList *self;
-    GSimpleAsyncResult *result;
-    gchar *path;
-} DeleteCallContext;
-
-static void
-delete_call_context_complete_and_free (DeleteCallContext *ctx)
-{
-    g_simple_async_result_complete (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
-    g_free (ctx->path);
-    g_free (ctx);
-}
-
 gboolean
-mm_call_list_delete_call_finish (MMCallList *self,
-                               GAsyncResult *res,
-                               GError **error)
+mm_call_list_delete_call_finish (MMCallList    *self,
+                                 GAsyncResult  *res,
+                                 GError       **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static guint
@@ -246,27 +223,31 @@ cmp_call_by_path (MMBaseCall *call,
 }
 
 static void
-delete_ready (MMBaseCall *call,
+delete_ready (MMBaseCall   *call,
               GAsyncResult *res,
-              DeleteCallContext *ctx)
+              GTask        *task)
 {
+    MMCallList *self;
+    const gchar *path;
     GError *error = NULL;
     GList *l;
 
+    self = g_task_get_source_object (task);
+    path = g_task_get_task_data (task);
     if (!mm_base_call_delete_finish (call, res, &error)) {
         /* We report the error */
-        g_simple_async_result_take_error (ctx->result, error);
-        delete_call_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* The CALL was properly deleted, we now remove it from our list */
-    l = g_list_find_custom (ctx->self->priv->list,
-                            ctx->path,
+    l = g_list_find_custom (self->priv->list,
+                            path,
                             (GCompareFunc)cmp_call_by_path);
     if (l) {
         g_object_unref (MM_BASE_CALL (l->data));
-        ctx->self->priv->list = g_list_delete_link (ctx->self->priv->list, l);
+        self->priv->list = g_list_delete_link (self->priv->list, l);
     }
 
     /* We don't need to unref the CALL any more, but we can use the
@@ -274,49 +255,45 @@ delete_ready (MMBaseCall *call,
      * during the async operation. */
     mm_base_call_unexport (call);
 
-    g_signal_emit (ctx->self,
+    g_signal_emit (self,
                    signals[SIGNAL_CALL_DELETED], 0,
-                   ctx->path);
+                   path);
 
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    delete_call_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 void
-mm_call_list_delete_call (MMCallList *self,
-                        const gchar *call_path,
-                        GAsyncReadyCallback callback,
-                        gpointer user_data)
+mm_call_list_delete_call (MMCallList          *self,
+                          const gchar         *call_path,
+                          GAsyncReadyCallback  callback,
+                          gpointer             user_data)
 {
-    DeleteCallContext *ctx;
     GList *l;
+    GTask *task;
 
     l = g_list_find_custom (self->priv->list,
                             (gpointer)call_path,
                             (GCompareFunc)cmp_call_by_path);
     if (!l) {
-        g_simple_async_report_error_in_idle (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_NOT_FOUND,
-                                             "No CALL found with path '%s'",
-                                             call_path);
+        g_task_report_new_error (self,
+                                 callback,
+                                 user_data,
+                                 mm_call_list_delete_call,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_NOT_FOUND,
+                                 "No CALL found with path '%s'",
+                                 call_path);
         return;
     }
 
     /* Delete all CALL parts */
-    ctx = g_new0 (DeleteCallContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->path = g_strdup (call_path);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_call_list_delete_call);
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, g_strdup (call_path), g_free);
 
     mm_base_call_delete (MM_BASE_CALL (l->data),
                         (GAsyncReadyCallback)delete_ready,
-                        ctx);
+                        task);
 }
 
 /*****************************************************************************/
@@ -337,9 +314,9 @@ MMCallList *
 mm_call_list_new (MMBaseModem *modem)
 {
     /* Create the object */
-    return g_object_new  (MM_TYPE_CALL_LIST,
-                          MM_CALL_LIST_MODEM, modem,
-                          NULL);
+    return g_object_new (MM_TYPE_CALL_LIST,
+                         MM_CALL_LIST_MODEM, modem,
+                         NULL);
 }
 
 static void
@@ -394,7 +371,7 @@ dispose (GObject *object)
     MMCallList *self = MM_CALL_LIST (object);
 
     g_clear_object (&self->priv->modem);
-    g_list_free_full (self->priv->list, (GDestroyNotify)g_object_unref);
+    g_list_free_full (self->priv->list, g_object_unref);
     self->priv->list = NULL;
 
     G_OBJECT_CLASS (mm_call_list_parent_class)->dispose (object);
